camera = {
	x = 0,
	y = 0,
	goal_x = 0,
	goal_y = 0,
	speed = 512,
	free = false,
	smooth = 5,
	layer = {}
}

function camera:set(scale)
	scale = scale or 1
	love.graphics.push()
	love.graphics.translate(-math.floor(self.x) * scale, -math.floor(self.y) * scale)
end

function camera:unset()
	love.graphics.pop()
end

function camera:newLayer(scale, func)
	self.layer[#self.layer + 1] = {
		scale = scale,
		draw = func
	}
	table.sort(self.layer, function(a,b) return a.scale < b.scale end)
end

function camera:update(dt)
	if not self.free then
		self.x = self.x + (self.goal_x - self.x) * (smooth_factor*dt)
		self.y = self.y + (self.goal_y - self.y) * (smooth_factor*dt)
	else
		if love.keyboard.isDown("up") then
			self.y = self.y - self.speed * dt
		elseif love.keyboard.isDown("down") then
			self.y = self.y + self.speed * dt
		end

		if love.keyboard.isDown("left") then
			self.x = self.x - self.speed * dt
		elseif love.keyboard.isDown("right") then
			self.x = self.x + self.speed * dt
		end
	end
end

function camera:draw()
	local bx, by = self.x, self.y
	for i,v in ipairs(self.layer) do
		self.x = bx * v.scale
		self.y = by * v.scale
		self:set()
		v:draw(self)
		self.unset()
	end
end

function camera:setPosition(x, y)
	self.goal_x = x
	self.goal_y = y
end

function camera:setRealPosition(x, y)
	x = x or self.x
	y = y or self.y
	self.x = x
	self.y = y
end

function camera:getMap()
	local start_x = math.ceil(camera.x / game.tileSize)
	local end_x = math.ceil((camera.x + screen.width) / game.tileSize)
	local start_y = math.ceil(camera.y / game.tileSize)
	local end_y = math.ceil((camera.y + screen.height) / game.tileSize)

	if start_x < 1 then
		start_x = 1
	elseif start_x > game.mapWidth then
		start_x = game.mapWidth
	end
	if end_x < 1 then
		end_x = 1
	elseif end_x > game.mapWidth then
		end_x = game.mapWidth
	end

	if start_y < 1 then
		start_y = 1
	elseif start_y >= game.mapHeight then
		start_y = game.mapHeight
	end
	if end_y < 1 then
		end_y = 1
	elseif end_y > game.mapHeight then
		end_y = game.mapHeight
	end
	return start_x, end_x, start_y, end_y
end

function camera:getScreenPosition(x, y)
	x = x or love.mouse.getX()
	y = y or love.mouse.getY()
	return x - self.x, y - self.y
end