local game = {firstStart = true}

function game:load()
	if self.firstStart then
		self.camX = 0
		self.camy = 0
		self.mapWidth = 1
		self.mapHeight = 1
		mapscreenWidth = 1
		mapscreenHeight = 1
		self.mapBoxes = 1
		self.tileSize = 16

		--Win screen
		self.wonScreen = {
			color = color[1],
			alpha = 1
		}

		self.t = 0
		self.val = 0
		self.currentMap = 1
		self.firstStart = false
		self.totalMoves = 0
		self:reset()
	else
		game:resize(screen.width, screen.height)
	end
end

function game:reset(nextMap)
	nextMap = nextMap or false
	local newGame = false
	if self.won then nextMap = true end
	if nextMap then
		sound:play("swish", soundVolume, false, 0.8)
		self.currentMap = self.currentMap + 1
		if self.currentMap > #maps then
			self.currentMap = 1
			newGame = true
		end
	else
		--sound:play("tick", soundVolume)
	end
	self.map, self.mapWidth, self.mapHeight, self.mapBoxes = map:loadMap(self.currentMap)
	--Finding player start position
	local sx, sy = 1, 1
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local t = self.map[2][y][x]
			if t then
				if t.tile == 5 then
					sx = x
					sy = y
					break
				end
			end
		end
	end
	player:spawn(sx, sy)
	self.moves = 0
	if newGame then
		self.totalMoves = 0
	end
	self.won = false
	self.winSoundPlayed = false
	player.currentQuad = 3
	love.resize(screen.width, screen.height)
end

function game:scaleMap()
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do

			local tile1 = self.map[1][y][x]
			local tile2 = self.map[2][y][x]
			if tile1 then
				tile1.x = (x - 1) * self.tileSize
				tile1.y = (y - 1) * self.tileSize
			end

			if tile2 then
				tile2.x = (x - 1) * self.tileSize
				tile2.y = (y - 1) * self.tileSize
			end
		end
	end
end

function game:setCamera(real)
	real = real or false

	self.camX = 0
	self.camY = 0
	if self.mapscreenWidth < screen.width then
		self.camX = (self.mapscreenWidth / 2) - (screen.width / 2)
	else
		self.camX = player.x - (screen.width / 2) + (self.tileSize / 2)
		if self.camX < 0 then
			self.camX = 0
		elseif self.camX + screen.width > self.mapscreenWidth then
			self.camX = self.mapscreenWidth - screen.width
		end
	end

	if self.mapscreenHeight < screen.height then
		self.camY = (self.mapscreenHeight / 2) - (screen.height / 2)
	else
		self.camY = player.y - (screen.height / 2) + (self.tileSize / 2)
		if self.camY < 0 then
			self.camY = 0
		elseif self.camY + screen.height > self.mapscreenHeight then
			self.camY = self.mapscreenHeight - screen.height
		end
	end

	if real then
		camera:setRealPosition(self.camX, self.camY)
	else
		camera:setPosition(self.camX, self.camY)
	end
end

function game:update(dt)
	self.t = self.t + dt
	if self.t > math.pi * 2 then t = 0 end
	self.val = math.sin(self.t)
	tile:update(dt)
	player:update(dt)

	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local box = self.map[2][y][x]
			local under = self.map[1][y][x]
			if box then
				box.x = box.x + ((box.gridX * self.tileSize) - box.x) * (smooth_factor * dt)
				box.y = box.y + ((box.gridY * self.tileSize) - box.y) * (smooth_factor * dt)
				box.travel = box.travel + (1 - box.travel) * (smooth_factor*dt)
				if box.onBelt then
					if under then
						if under.tile == 6 then
							self:moveBox(x, y, x, y + 1, true)
						elseif under.tile == 7 then
							self:moveBox(x, y, x - 1, y, true)
						elseif under.tile == 8 then
							self:moveBox(x, y, x, y - 1, true)
						elseif under.tile == 9 then
							self:moveBox(x, y, x + 1, y, true)
						end
					end
				end
			end
		end
	end

	if self.won then
		self.wonScreen.alpha = self.wonScreen.alpha + (1 - self.wonScreen.alpha) * (smooth_factor * dt)
		if not self.winSoundPlayed then
			sound:play("swish", soundVolume, false, 0.6)
			sound:play("win", soundVolume)
			self.winSoundPlayed = true
		end
	else
		self.wonScreen.alpha = self.wonScreen.alpha + (0 - self.wonScreen.alpha) * (smooth_factor * dt)
	end

	self:setCamera()
	camera:update(dt)
end

function game:moveBox(x, y, newX, newY, travel)
	travel = travel or false
	local move = true

	if newX == player.gridX and newY == player.gridY then
		move = false
	end

	if travel then
		if self.map[2][y][x].travel <= 0.99 then 
			move = false 
		end
	end

	--LAYER 1 collisions
	if self.map[1][newY][newX] then
		if self.map[1][newY][newX].tile < 2 then
			move = false
		end
	end

	--LAYER 2 collisions
	if self.map[2][newY][newX] then
		if self.map[2][newY][newX].tile == 4 then
			move = false
		end
	end

	local onTarget = false
	local onBelt = false
	if self.map[1][newY][newX].tile == 3 then
		onTarget = true
	end
	if self.map[1][newY][newX].tile > 5 and self.map[1][newY][newX].tile < 10 then
		onBelt = true
	end
	
	if move then
		if self.map[1][y][x].tile < 5 or self.map[1][newY][newX].tile < 5 then
			sound:play("drag", soundVolume, true)
		end
		if onTarget and not self.map[2][y][x].onTarget then
			sound:play("target", soundVolume) 
		end

		local rep = self.map[2][y][x]
		rep.gridX = newX - 1
		rep.gridY = newY - 1
		rep.onTarget = onTarget
		rep.onBelt = onBelt
		rep.travel = 0

		self.map[2][newY][newX] = rep
		self.map[2][y][x] = false

		game:checkIfWon()
	end
end

function game:draw()
	self.gameCanvas:clear()
	love.graphics.setCanvas(self.gameCanvas)

	camera:set()

	local start_x, end_x, start_y, end_Y = camera:getMap()

	for y=start_y, end_Y do
		for x=start_x, end_x do
			local t = self.map[1][y][x]
			if t then
				love.graphics.setColor(255, 255, 255, 255)
				tile:draw(t.tile, t.x, t.y)
			end
		end
	end

	for y=start_y, end_Y do
		for x=start_x, end_x do
			local t = self.map[2][y][x]

			if t then
				love.graphics.setColor(255, 255, 255, 255)
				tile:draw(t.tile, t.x, t.y)
				if t.onTarget then
					love.graphics.setBlendMode("additive")
					love.graphics.setColor(255, 255, 255, 126)
					tile:draw(t.tile, t.x, t.y)
					love.graphics.setBlendMode("alpha")
				end
			end
		end
	end

	player:draw()

	camera:unset()

	if not self.won then
		love.graphics.setColor(0, 0, 0, 126)
		local y = screen.height - (font.small:getAscent() - font.small:getDescent()) - 8
		love.graphics.rectangle("fill", 0, y - 8, screen.width, y)
		love.graphics.setColor(color[4])
		love.graphics.setFont(font.small)
		love.graphics.printf("Moves: "..self.moves, 8, y, screen.width, "left")
		love.graphics.printf("Level: "..self.currentMap.."/"..#maps, -8, y, screen.width, "right")
	end

	self.textCanvas:clear()
	love.graphics.setCanvas(self.textCanvas)
	if self.wonScreen.alpha > 0 then

		love.graphics.setColor(color[4])
		setAlpha(self.wonScreen.alpha)
		love.graphics.setFont(font.medium)
		local beatString = "You beat level "..self.currentMap.."!"
		if self.currentMap == #maps then
			beatString = "You beat all the levels! Congratulations."
		end
		love.graphics.printf(beatString, 0, math.floor(screen.height * 0.15), screen.width, "center")
		love.graphics.setFont(font.medium)
		local moveString = "Moves: "..self.moves
		if self.currentMap == #maps then
			moveString = "Level moves: "..self.moves..", Total moves: "..game.totalMoves
		end
		love.graphics.printf(moveString, 0, math.floor(screen.height * 0.31), screen.width, "center")
		love.graphics.setFont(font.small)
		local resetString = "Press 'r' to go to the next level"
		if self.currentMap == #maps then
			resetString = "Press 'r' to start a new game"
		end
		love.graphics.printf(resetString, 0, math.floor(screen.height * 0.9), screen.width, "center")
	end
	love.graphics.setCanvas()

	--GAME
	love.graphics.setColor(255, 255, 255, 255)
	setAlpha(1 - self.wonScreen.alpha)
	love.graphics.draw(self.gameCanvas, (screen.width / 2), screen.height / 2, 0, 1 - self.wonScreen.alpha, 1 - self.wonScreen.alpha, screen.width / 2, screen.height / 2)

	--TEXT
	love.graphics.setColor(self.wonScreen.color)
	setAlpha(self.wonScreen.alpha)
	--love.graphics.rectangle("fill", 0, 0, screen.width, screen.height)

	love.graphics.setColor(255, 255, 255, 255 * self.wonScreen.alpha)
	love.graphics.draw(self.textCanvas, (screen.width / 2), screen.height / 2, 0, 2 - self.wonScreen.alpha, 2 - self.wonScreen.alpha, screen.width / 2, screen.height / 2)
end

function game:keypressed(key)
	if not console:getVisible() then
		if not self.won then
			if key == "w" or key == "up" then
				player:move("up")
			elseif key == "s" or key == "down" then
				player:move("down")
			elseif key == "d"  or key == "right" then
				player:move("right")
			elseif key == "a" or key == "left" then
				player:move("left")
			elseif key == "escape" then
				gState:selectState("menu")
				sound:play("tick", soundVolume, false, 0.8)
			end
		end

		if key == "r" then
			self:reset()
		end
	end
end

function game:checkIfWon()
	local onTarget = 0
	for y=1, self.mapHeight do
		for x=1, self.mapWidth do
			local box = self.map[2][y][x]
			if box then
				if box.onTarget then
					onTarget = onTarget + 1
				end
			end
		end
	end
	if onTarget == self.mapBoxes then
		self.won = true
	end
end

function game:resize(w, h)
	self.mapscreenWidth = self.mapWidth * self.tileSize
	self.mapscreenHeight = self.mapHeight * self.tileSize
	self:setCamera(true)
	local msaa = love.graphics.getSystemLimit("canvasfsaa")
	console:print(msaa)
	self.gameCanvas = love.graphics.newCanvas(screen.width, screen.height, "normal", msaa)
	self.textCanvas = love.graphics.newCanvas(screen.width, screen.height, "normal", msaa)
	player:spawn()
	game:scaleMap()
end

return game