map = {}

function map:load()
	self.map = {}
	for i,v in ipairs(maps) do
		local rawMap, width, height = self:loadMapFile("src/map/"..v..".txt")
		local map, boxes = self:format(rawMap)
		self.map[i] = {map = map, mapWidth = width, mapHeight = height, boxes = boxes}
	end
end

function map:loadMapFile(path)
	if love.filesystem.exists(path) then
		local map = {}
		local str = {}
		local mapWidth = 1
		local mapHeight = 1
		local currentLayer = 1

		for line in love.filesystem.lines(path) do
			if line:sub(1, 9) == "tileswide" then
				mapWidth = tonumber(line:sub(10))
			elseif line:sub(1, 9) == "tileshigh" then
				mapHeight = tonumber(line:sub(10))
			elseif line:sub(1, 5) == "layer" then
				currentLayer = tonumber(line:sub(6)) + 1
				str[currentLayer] = "return {"
			elseif tonumber(line:sub(1, 1)) or tonumber(line:sub(1, 2)) then
				str[currentLayer] = str[currentLayer].."{"..line:sub(1, -2).."},"
			end
		end

		for i,v in ipairs(str) do
			v = v:sub(1, -2)
			v = v.."}"
			local f, e = loadstring(v)
			map[i] = f()
		end

		return map, mapWidth, mapHeight
	else
		error("'"..path.."' not found")
	end
end

function map:format(rawMap)
	local forMap = {{}, {}}
	local boxes = 0
	for y=1, #rawMap[1] do
		forMap[1][y] = {}
		forMap[2][y] = {}
		for x=1, #rawMap[1][1] do
			local tile1 = rawMap[1][y][x]
			local tile2 = rawMap[2][y][x]
			if tile1 then
				if tile1 + 1 > 0 then
					forMap[1][y][x] = {
						tile = tile1 + 1,
						gridX = x - 1,
						gridY = y - 1,
						x = (x - 1) * game.tileSize,
						y = (y - 1) * game.tileSize
					}
				else 
					forMap[1][y][x] = false
				end
			end
			if tile2 then
				if tile2 + 1 > 0 then
					if tile2 + 1 == 4 then
						boxes = boxes + 1
					end
					local onBelt = false
					if tile1 > 4 and tile1 < 9 then
						onBelt = true
					end
					forMap[2][y][x] = {
						tile = tile2 + 1,
						gridX = x - 1,
						gridY = y - 1,
						x = (x - 1) * game.tileSize,
						y = (y - 1) * game.tileSize,
						travel = 1,
						onTarget = false,
						onBelt = onBelt
					}
				else
					forMap[2][y][x] = false
				end
			end
		end
	end
	return forMap, boxes
end

function map:loadMap(map)
	if self.map[map] then
		return copy(self.map[map].map), self.map[map].mapWidth, self.map[map].mapHeight, self.map[map].boxes
	end
end