local menu = {}

function menu:load()
	self.screen = {
		main = {
			selectedItem = 1,
			text = "Main menu",
			items = {
				{options = {"Start"}, selected = 1},
				{options = {"Options"}, selected = 1},
				{options = {"Exit"}, selected = 1},
			}
		},
		options = {
			selectedItem = 1,
			text = "options",
			items = {
				{name = "Screen resolution", options = {}, selected = 1},
				{name = "Fullscreen", options = {"enabled", "disabled"}, selected = 2},
				{name = "Vsync", options = {"enabled", "disabled"}, selected = 1},
				{name = "SoundFX", options = {"off", "10%", "20%", "30%", "40%", "50%", "60%", "70%", "80%", "90%", "100%"}, selected = 1},
				{name = "Music", options = {"off", "10%", "20%", "30%", "40%", "50%", "60%", "70%", "80%", "90%", "100%"}, selected = 1},
				{name = "Background", options = {"on", "off"}, selected = 1},
				{options = {"Apply"}, selected = 1},
				{options = {"Back"}, selected = 1}
			}
		}
	}
	self.selectedScreen = "main"

	--Start button
	if not game.firstStart then
		self.screen.main.items[1].options[1] = "Continue"
		self.screen.main.items[2].options[1] = "New game"
		self.screen.main.items[3].options[1] = "Options"
		self.screen.main.items[4] = {options = {"Exit"}, selected = 1}
	end

	--Loading resolutions
	local res = love.window.getFullscreenModes()
	table.sort(res, function(a, b) return a.width*a.height > b.width*b.height end)
	for i=#res, 1, -1 do
		v = res[i]
		self.screen.options.items[1].options[#self.screen.options.items[1].options + 1] = v.width.."x"..v.height
	end

	--Loading settings
	--Resolution
	local currentRes = screen.width.."x"..screen.height
	local onList = false
	for i,v in ipairs(self.screen.options.items[1].options) do
		if v == currentRes then
			self.screen.options.items[1].selected = i
			onList = true
			break
		end
	end
	if not onList then
		self.screen.options.items[1].options[#self.screen.options.items[1].options + 1] = screen.width.."x"..screen.height
		self.screen.options.items[1].selected = #self.screen.options.items[1].options
	end

	--Fullscreen
	if screen.fullscreen then
		self.screen.options.items[2].selected = 1
	else
		self.screen.options.items[2].selected = 2
	end

	--vSync
	if screen.vsync then
		self.screen.options.items[3].selected = 1
	else
		self.screen.options.items[3].selected = 2
	end

	--Sound Volume
	local snd = (soundVolume * 100).."%"
	local onList = false
	for i,v in ipairs(self.screen.options.items[4].options) do
		if v == snd then
			self.screen.options.items[4].selected = i
			onList = true
			break
		end
	end
	if not onList then
		self.screen.options.items[4].selected = 1
	end

	--Music Volume
	local snd = (musicVolume * 100).."%"
	local onList = false
	for i,v in ipairs(self.screen.options.items[5].options) do
		if v == snd then
			self.screen.options.items[5].selected = i
			onList = true
			break
		end
	end
	if not onList then
		self.screen.options.items[5].selected = 1
	end

	--background
	if showBackground then
		self.screen.options.items[6].selected = 1
	else
		self.screen.options.items[6].selected = 2
	end

	love.resize(screen.width, screen.height)

	self.message = {
		text = "",
		time = 0,
		alpha = 0,
		visible = false
	}
end

function menu:update(dt)
	if self.message.time > 0 then
		self.message.alpha = self.message.alpha + (1 - self.message.alpha) * (smooth_factor * dt)
		self.message.time = self.message.time - dt
		if self.message.time < 0 then
			self.message.time = 0
		end
	else
		self.message.alpha = self.message.alpha + (0 - self.message.alpha) * (smooth_factor * dt)
	end
end

function menu:draw()
	love.graphics.setFont(font.large)
	love.graphics.setColor(color[4])
	love.graphics.printf("Sokoban", 16, math.floor(screen.height * 0.05), screen.width, "left")

	--Items
	love.graphics.setFont(font.small)
	if self.selectedScreen == "main" then
		love.graphics.setFont(font.medium)
	end
	for i,v in ipairs(self.screen[self.selectedScreen].items) do
		love.graphics.setColor(color[2])
		if self.screen[self.selectedScreen].selectedItem == i then
			love.graphics.setColor(color[4])
		end
		local str = v.options[v.selected]
		if v.name then str = v.name..": "..str end
		love.graphics.printf(str, 16, math.floor(screen.height * 0.2) + (self.itemOffset * i), screen.width, "left")
	end

	--Message
	if self.selectedScreen == "options" then
		if self.message.alpha > 0 then
			love.graphics.setFont(font.small)
			love.graphics.setColor(color[4])
			setAlpha(self.message.alpha)
			love.graphics.printf(self.message.text, 0, screen.height - (font.small:getAscent() - font.small:getDescent()) - 16, screen.width, "center")
		end
	end
end

function menu:keypressed(key)
	if key == "down" then
		self.screen[self.selectedScreen].selectedItem = self.screen[self.selectedScreen].selectedItem + 1
		if self.screen[self.selectedScreen].selectedItem > #self.screen[self.selectedScreen].items then
			self.screen[self.selectedScreen].selectedItem = 1
		end
		sound:play("tick", soundVolume)
	elseif key == "up" then
		self.screen[self.selectedScreen].selectedItem = self.screen[self.selectedScreen].selectedItem - 1
		if self.screen[self.selectedScreen].selectedItem < 1 then
			self.screen[self.selectedScreen].selectedItem = #self.screen[self.selectedScreen].items
		end
		sound:play("tick", soundVolume)
	elseif key == "right" then
		local item = self.screen[self.selectedScreen].items[self.screen[self.selectedScreen].selectedItem]
		item.selected = item.selected + 1
		if item.selected > #item.options then
			item.selected = 1
		end
		sound:play("tick", soundVolume)
	elseif key == "left" then
		local item = self.screen[self.selectedScreen].items[self.screen[self.selectedScreen].selectedItem]
		item.selected = item.selected - 1
		if item.selected < 1 then
			item.selected = #item.options
		end
		sound:play("tick", soundVolume)
	elseif key == "escape" then
		if self.selectedScreen == "options" then
			self.selectedScreen = "main"
			sound:play("tick", soundVolume, false, 0.8)
		elseif self.selectedScreen == "main" then
			if not gState.states["game"].firstStart then
				gState:selectState("game")
				sound:play("tick", soundVolume, false, 0.8)
			end
		end
	elseif key == "return" then
		local mScreen = self.selectedScreen
		local item = self.screen[self.selectedScreen].items[self.screen[self.selectedScreen].selectedItem]
		local option = item.options[item.selected]

		--MAIN MENU
		if option == "Start" or option == "Continue" then
			gState:selectState("game")
			sound:play("tick", soundVolume, false, 0.8)
			if option == "Start" then
				sound:play("swish", soundVolume, false, 0.8)
			end
		elseif option == "New game" then
			gState.states["game"].firstStart = true
			gState:selectState("game")
			sound:play("tick", soundVolume, false, 0.8)
			sound:play("swish", soundVolume, false, 0.8)
		elseif option == "Options" then
			self.selectedScreen = "options"
			sound:play("tick", soundVolume, false, 0.8)
		elseif option == "Exit" then
			love.event.push("quit")
		elseif option == "Back" then
			if mScreen == "options" then
				self.selectedScreen = "main"
				sound:play("tick", soundVolume, false, 0.8)
			end
		elseif option == "Apply" then
			sound:play("tick", soundVolume, false, 0.8)
			--resolution
			local res = self.screen.options.items[1].options[self.screen.options.items[1].selected]
			local s = res:find("x", 1, true)
			local screenWidth = tonumber(res:sub(1, s-1))
			local screenHeight = tonumber(res:sub(s+1))

			--Fullscreen
			local fScreen = self.screen.options.items[2].options[self.screen.options.items[2].selected]
			local fullscreen = true
			if fScreen == "disabled" then
				fullscreen = false
			end
			screen.fullscreen = fullscreen

			--vSync
			local vs = self.screen.options.items[3].options[self.screen.options.items[3].selected]
			local vsync = true
			if vs == "disabled" then
				vsync = false
			end

			--soundVolume
			local snd = self.screen.options.items[4].options[self.screen.options.items[4].selected]
			if snd ~= "off" then
				local vol = tonumber(snd:sub(1, -2))
				soundVolume = vol * 0.01
			else
				soundVolume = 0
			end

			--musicVolume
			local mus = self.screen.options.items[5].options[self.screen.options.items[5].selected]
			if mus ~= "off" then
				local vol = tonumber(mus:sub(1, -2))
				musicVolume = vol * 0.01
			else
				musicVolume = 0
				sound:stop("song")
			end

			sound:setVolume("song", musicVolume)
			if not sound:isPlaying("song") then sound:play("song", musicVolume, false, 1, true, false) end


			--background
			local bg = self.screen.options.items[6].options[self.screen.options.items[6].selected]
			showBackground = true
			if bg == "off" then
				showBackground = false
			end

			if configFile then
				--------------------------------------------------
				if configFile:exists("screenWidth") then
					configFile:setValue("screenWidth", screenWidth)
				else
					configFile:addItem("screenWidth", screenWidth)
				end
				--------------------------------------------------
				if configFile:exists("screenHeight") then
					configFile:setValue("screenHeight", screenHeight)
				else
					configFile:addItem("screenHeight", screenHeight)
				end
				--------------------------------------------------
				if configFile:exists("fullscreen") then
					configFile:setValue("fullscreen", fullscreen)
				else
					configFile:addItem("fullscreen", fullscreen)
				end
				--------------------------------------------------
				if configFile:exists("vsync") then
					configFile:setValue("vsync", vsync)
				else
					configFile:addItem("vsync", vsync)
				end
				--------------------------------------------------
				if configFile:exists("soundVolume") then
					configFile:setValue("soundVolume", soundVolume)
				else
					configFile:addItem("soundVolume", soundVolume)
				end
				--------------------------------------------------
				if configFile:exists("musicVolume") then
					configFile:setValue("musicVolume", musicVolume)
				else
					configFile:addItem("musicVolume", musicVolume)
				end
				--------------------------------------------------
				if configFile:exists("showBackground") then
					configFile:setValue("showBackground", showBackground)
				else
					configFile:addItem("showBackground", showBackground)
				end
				--------------------------------------------------
				configFile:save()
			else
				configFile:addItem("screenWidth", screenWidth)
				configFile:addItem("screenHeight", screenHeight)
				configFile:addItem("fullscreen", fullscreen)
				configFile:addItem("vsync", vsync)
				configFile:addItem("soundVolume", soundVolume)
				configFile:addItem("musicVolume", musicVolume)
				configFile:addItem("showBackground", showBackground)
				configFile:save()
			end

			love.window.setMode(screenWidth, screenHeight, {fullscreen = fullscreen, vsync = vsync})
			love.resize(screenWidth, screenHeight)
			self.message.text = "Settings applied"
			self.message.time = 3
		end
	end
end

function menu:resize(w, h)
	self.itemOffset = math.floor(h * 0.08)
end

return menu