player = {}

function player:load()
	self.img = love.graphics.newImage("src/img/player.png")
	self.quadCount = 4
	self.quadSize = 32
	self.img:setFilter("nearest", "nearest")
	self.quad = {}
	for i=1, self.quadCount do
		self.quad[i] = love.graphics.newQuad(self.quadSize * (i - 1), 0, self.quadSize, self.quadSize, self.img:getWidth(), self.img:getHeight())
	end
	self.travel = 1
	self.travelDirection = "right"
	self.stretchFactor = 4
	self.currentQuad = 1
	game = gState.states["game"]
end

function player:spawn(x, y)
	x = x or self.gridX
	y = y or self.gridY
	self.gridX = x
	self.gridY = y
	self.x = (self.gridX - 1) * game.tileSize 
	self.y = (self.gridY - 1) * game.tileSize
end

function player:update(dt)
	self.x = self.x + ((self.gridX - 1) * game.tileSize - self.x) * (smooth_factor * dt)
	self.y = self.y + ((self.gridY - 1) * game.tileSize - self.y) * (smooth_factor * dt)
	self.travel = self.travel + (1 - self.travel) * (smooth_factor*dt)
end

function player:draw(offsetX, offsetY)
	love.graphics.setColor(255, 255, 255, 255)
	if self.travelDirection == "right" or self.travelDirection == "left" then
		love.graphics.draw(self.img, self.quad[self.currentQuad], self.x + (game.tileSize / 2), self.y + (game.tileSize  / 2), 0, (game.tileSize / self.quadSize) + (self.stretchFactor * (1-self.travel)), game.tileSize / self.quadSize, self.quadSize / 2, self.quadSize / 2)
	else
		love.graphics.draw(self.img, self.quad[self.currentQuad], self.x + (game.tileSize / 2), self.y + (game.tileSize  / 2), 0, (game.tileSize / self.quadSize), game.tileSize / self.quadSize + ((self.stretchFactor / 3) * (1-self.travel)), self.quadSize / 2, self.quadSize / 2)
	end
end

function player:move(direction)
	local nx, ny = self.gridX, self.gridY
	local nbx, nby = self.gridX, self.gridY

	self.travelDirection = direction

	if direction == "right" then
		nx = nx + 1
		nbx = nbx + 2
		self.currentQuad = 1
	elseif direction == "left" then
		nx = nx - 1
		nbx = nbx - 2
		self.currentQuad = 2
	elseif direction == "down" then
		ny = ny + 1
		nby = nby + 2
		self.currentQuad = 3
	elseif direction == "up" then
		ny = ny - 1
		nby = nby - 2
		self.currentQuad = 4
	end

	local nTile1 = game.map[1][ny][nx]
	local nTile2 = game.map[2][ny][nx]
	local step = false
	local box = false
	target = false
	--Layer 1 collisions
	if nTile1 then
		if nTile1.tile > 1 then
			if nTile2 then
				if nTile2.tile == 4 then
					local moveBox = false
					local nTile1 = game.map[1][nby][nbx]
					local nTile2 = game.map[2][nby][nbx]
					if nTile1 then
						if nTile1.tile > 1 then
							moveBox = true
						end
					end
					if nTile2 then
						if nTile2.tile == 4 then
							moveBox = false
						end
					end

					if moveBox then
						box = true
						local onTarget = false
						if game.map[1][nby][nbx].tile == 3 then
							onTarget = true
							if game.map[1][ny][nx].tile ~= 3 then
								target = true
							end
						end
						game:moveBox(nx, ny, nbx, nby)
						step = true
					else
						step = false
					end
				else
					step = true
				end
			else
				step = true
			end
		end
	end

	if step then
		self.gridX = nx
		self.gridY = ny
		self.travel = 0
		game.moves = game.moves + 1
		game.totalMoves = game.totalMoves + 1
		game:checkIfWon()
		sound:play("swish", soundVolume, true)
	end
end

--[[
1 = WALL
2 = FLOOR
3 = TARGET
4 = BOX
5 = PLAYER START

if nTile2.tile == 4 then
	local moveBox = false
	local nTile1 = game.map[1][nby][nbx]
	local nTile2 = game.map[2][nby][nbx]
	if nTile1 then
		if nTile1.tile > 1 then
			moveBox = true
		end
	end
	if nTile2 then
		if nTile2.tile == 4 then
			moveBox = false
		end
	end

	if moveBox then
		box = true
		local onTarget = false
		if game.map[1][nby][nbx].tile == 3 then
			onTarget = true
			if game.map[1][ny][nx].tile ~= 3 then
				target = true
			end
		end
		local rep = {}
		rep.tile = 4
		rep.gridX = nbx - 1
		rep.gridY = nby - 1
		rep.x = game.map[2][ny][nx].x
		rep.y = game.map[2][ny][nx].y
		rep.onTarget = onTarget

		game.map[2][nby][nbx] = rep
		game.map[2][ny][nx] = false

		step = true
	end

]]